@@ -3,7 +3,7 @@
  *
  * This software is published under the terms of the Apache Software
  * License version 1.1, a copy of which has been included with this
- * distribution in the LICENSE.APL file.  */
+ * distribution in the LICENSE.txt file.  */
 
 package org.apache.log4j;
 
@@ -17,29 +17,16 @@ import org.apache.log4j.helpers.LogLog;
 
 
 /** 
-   Abstract superclass of the other appenders in the package.
-   
-   This class provides the code for common functionality, such as
-   support for threshold filtering and support for general filters.
-
-   @since 0.8.1
-   @author Ceki G&uuml;lc&uuml; */
+ * Abstract superclass of the other appenders in the package.
+ *  
+ *  This class provides the code for common functionality, such as
+ *  support for threshold filtering and support for general filters.
+ *
+ * @since 0.8.1
+ * @author Ceki G&uuml;lc&uuml; 
+ * */
 public abstract class AppenderSkeleton implements Appender, OptionHandler {
 
-  /**
-     A string constant used in naming the option for setting the
-     threshold for the appender. See also {@link #setThreshold
-     setThreshold} method. Current value of this string constant is
-     <b>Threshold</b>.
-
-     @deprecated Options are now handled using the JavaBeans paradigm.
-     This constant is not longer needed and will be removed in the
-     <em>near</em> term.
-     
-  */
-  public static final String THRESHOLD_OPTION = "Threshold";
-
-
   /** The layout variable does not need to be set if the appender
       implementation has its own layout. */
   protected Layout layout;
@@ -48,7 +35,7 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
   protected String name;
 
   /**
-     There is no priority threshold filtering by default.  */
+     There is no level threshold filtering by default.  */
   protected Priority threshold;
 
   /** 
@@ -113,11 +100,10 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
   }
 
   /**
-     Finalize this appender by calling the imlenentation's
+     Finalize this appender by calling the derived class'
      <code>close</code> method.
 
-     @since 0.8.4
-  */
+     @since 0.8.4 */
   public
   void finalize() {
     // An appender might be closed then garbage collected. There is no
@@ -182,23 +168,7 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
   }
 
   /**
-     Returns the string array {{@link #THRESHOLD_OPTION}}.
-
-     <p>Configurable appenders must override this method to return the
-     additional options they accept.  
-     
-     @deprecated We now use JavaBeans introspection to configure
-     components. Options strings are no longer needed.
-   */
-  public
-  String[] getOptionStrings() {
-    return new String[] {THRESHOLD_OPTION};
-  }
-
-  
-
-  /**
-     Returns this appenders threshold priority. See the {@link
+     Returns this appenders threshold level. See the {@link
      #setThreshold} method for the meaning of this option.
      
      @since 1.1 */
@@ -209,7 +179,7 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
 
 
   /**
-     Check whether the message priority is below the appender's
+     Check whether the message level is below the appender's
      threshold. If there is no threshold set, then the return value is
      always <code>true</code>.
 
@@ -221,19 +191,19 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
 
 
   /**
-     This method performs threshold checks and invokes filters before
-     delegating actual logging to the subclasses specific {@link
-     AppenderSkeleton#append} method.
-
-   */
+    * This method performs threshold checks and invokes filters before
+    * delegating actual logging to the subclasses specific {@link
+    * AppenderSkeleton#append} method.
+    * */
   public
   synchronized 
   void doAppend(LoggingEvent event) {
     if(closed) {
       LogLog.error("Attempted to append to closed appender named ["+name+"].");
+      return;
     }
-
-    if(!isAsSevereAsThreshold(event.priority)) {
+    
+    if(!isAsSevereAsThreshold(event.getLevel())) {
       return;
     }
 
@@ -289,22 +259,11 @@ public abstract class AppenderSkeleton implements Appender, OptionHandler {
 
 
   /**
-     @deprecated Use the setter method for the option directly instead
-     of the generic <code>setOption</code> method. 
-  */
-  public
-  void setOption(String key, String value) {
-    if(key.equalsIgnoreCase(THRESHOLD_OPTION)) {
-      threshold = Priority.toPriority(value);
-    }
-  }
-  
-  /**
-     Set the threshold priority. All log events with lower priority
-     than the threshold priority are ignored by the appender.
+     Set the threshold level. All log events with lower level
+     than the threshold level are ignored by the appender.
      
      <p>In configuration files this option is specified by setting the
-     value of the <b>Threshold</b> option to a priority
+     value of the <b>Threshold</b> option to a level
      string, such as "DEBUG", "INFO" and so on.
      
      @since 0.8.3 */
