@@ -1,7 +1,13 @@
-//      Copyright 1996-1999, International Business Machines 
-//      Corporation. All Rights Reserved.
-//
-//      See the LICENCE file for the terms of usage and distribution.
+/*
+ * Copyright (C) The Apache Software Foundation. All rights reserved.
+ *
+ * This software is published under the terms of the Apache Software License
+ * version 1.1, a copy of which has been included  with this distribution in
+ * the LICENSE.txt file.
+ */
+
+//      Contributors:      Dan Milstein 
+//                         Ray Millard
 
 package org.apache.log4j;
 
@@ -80,7 +86,7 @@ import org.apache.log4j.helpers.LogLog;
    thread, in particular to a child.
    
    @author Ceki G&uuml;lc&uuml;
-   @since log4j v0.7.0
+   @since 0.7.0
   
 */
  
@@ -186,7 +192,8 @@ public class NDC {
 
 
   /**
-     Used when printing the diagnostic context.
+     <font color="#FF4040"><b>Never use this method directly, use the {@link
+     org.apache.log4j.spi.LoggingEvent#getNDC} method instead</b></font>.
   */
   static
   public
@@ -199,10 +206,10 @@ public class NDC {
   }
   
   /**
-     Get the current nesting depth of this diagnostic context.
-
-     @see #setMaxDepth
-     @since 0.7.5
+   * Get the current nesting depth of this diagnostic context.
+   *
+   * @see #setMaxDepth
+   * @since 0.7.5
    */
   public
   static
@@ -225,7 +232,7 @@ public class NDC {
     
     synchronized(ht) {
       // Avoid calling clean-up too often.
-      if(++pushCounter >= REAP_THRESHOLD) {
+      if(++pushCounter <= REAP_THRESHOLD) {
 	return; // We release the lock ASAP.
       } else {
 	pushCounter = 0; // OK let's do some work.
@@ -233,13 +240,13 @@ public class NDC {
 
       int misses = 0;
       v = new Vector(); 
-      Enumeration enum = ht.keys();
+      Enumeration enumeration = ht.keys();
       // We give up after 4 straigt missses. That is 4 consecutive
       // inspected threads in 'ht' that turn out to be alive.
       // The higher the proportion on dead threads in ht, the higher the
       // chances of removal.
-      while(enum.hasMoreElements() && (misses <= 4)) {
-	Thread t = (Thread) enum.nextElement();
+      while(enumeration.hasMoreElements() && (misses <= 4)) {
+	Thread t = (Thread) enumeration.nextElement();
 	if(t.isAlive()) {
 	  misses++;
 	} else {
