@@ -3,24 +3,22 @@
  *
  * This software is published under the terms of the Apache Software
  * License version 1.1, a copy of which has been included with this
- * distribution in the LICENSE.APL file.  */
+ * distribution in the LICENSE.txt file.  */
 
 package org.apache.log4j;
 
 import java.io.IOException;
 import java.io.Writer;
-import java.io.FileWriter;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 
 import org.apache.log4j.spi.ErrorHandler;
-import org.apache.log4j.spi.ErrorCode;
 import org.apache.log4j.spi.LoggingEvent;
-import org.apache.log4j.helpers.OptionConverter;
 import org.apache.log4j.helpers.QuietWriter;
 import org.apache.log4j.helpers.LogLog;
 
 // Contibutors: Jens Uwe Pipka <jens.pipka@gmx.de>
+//              Ben Sandee
 
 /**
    WriterAppender appends log events to a {@link java.io.Writer} or an
@@ -30,18 +28,6 @@ import org.apache.log4j.helpers.LogLog;
    @since 1.1 */
 public class WriterAppender extends AppenderSkeleton {
 
-  /**
-     A string constant used in naming the option for immediate
-     flushing of the output stream at the end of each append
-     operation. Current value of this string constant is
-     <b>ImmediateFlush</b>.
-
-     <p>Note that all option keys are case sensitive.     
-
-     @deprecated We now use JavaBeans introspection to configure
-     components. Options strings are no longer needed. 
-  */
-  public static final String IMMEDIATE_FLUSH_OPTION = "ImmediateFlush";
 
   /**
      Immediate flush means that the underlying writer or output stream
@@ -58,13 +44,20 @@ public class WriterAppender extends AppenderSkeleton {
   */
   protected boolean immediateFlush = true;
 
+  /**
+     The encoding to use when opening an InputStream.  <p>The
+     <code>encoding</code> variable is set to <code>null</null> by
+     default which results in the utilization of the system's default
+     encoding.  */
+  protected String encoding;
+
   /**
      This is the {@link QuietWriter quietWriter} where we will write
-     to. 
+     to.
   */
   protected QuietWriter qw;
 
-  
+
   /**
      This default constructor does nothing.  */
   public
@@ -79,7 +72,7 @@ public class WriterAppender extends AppenderSkeleton {
   WriterAppender(Layout layout, OutputStream os) {
     this(layout, new OutputStreamWriter(os));
   }
-  
+
   /**
      Instantiate a WriterAppender and set the output destination to
      <code>writer</code>.
@@ -97,7 +90,7 @@ public class WriterAppender extends AppenderSkeleton {
      <code>true</code>, the appender will flush at the end of each
      write. This is the default behavior. If the option is set to
      <code>false</code>, then the underlying stream can defer writing
-     to physical medium to a later time. 
+     to physical medium to a later time.
 
      <p>Avoiding the flush operation at the end of each append results in
      a performance gain of 10 to 20 percent. However, there is safety
@@ -123,13 +116,13 @@ public class WriterAppender extends AppenderSkeleton {
      Does nothing.
   */
   public
-  void activateOptions() {    
+  void activateOptions() {
   }
 
 
   /**
      This method is called by the {@link AppenderSkeleton#doAppend}
-     method. 
+     method.
 
      <p>If the output stream exists and is writable then write a log
      statement to the output stream. Otherwise, write a single warning
@@ -137,7 +130,7 @@ public class WriterAppender extends AppenderSkeleton {
 
      <p>The format of the output will depend on this appender's
      layout.
-     
+
   */
   public
   void append(LoggingEvent event) {
@@ -155,11 +148,11 @@ public class WriterAppender extends AppenderSkeleton {
       return;
     }
     subAppend(event);
-   } 
+   }
 
   /**
      This method determines if there is a sense in attempting to append.
-     
+
      <p>It checks whether there is a set output target and also if
      there is a set layout. If these checks fail, then the boolean
      value <code>false</code> is returned. */
@@ -171,11 +164,11 @@ public class WriterAppender extends AppenderSkeleton {
     }
 
     if(this.qw == null) {
-      errorHandler.error("No output stream or file set for the appender named ["+ 
+      errorHandler.error("No output stream or file set for the appender named ["+
 			name+"].");
       return false;
     }
-    
+
     if(this.layout == null) {
       errorHandler.error("No layout set for the appender named ["+ name+"].");
       return false;
@@ -187,7 +180,7 @@ public class WriterAppender extends AppenderSkeleton {
   /**
      Close this appender instance. The underlying stream or writer is
      also closed.
-     
+
      <p>Closed appenders cannot be reused.
 
      @see #setWriter
@@ -203,40 +196,60 @@ public class WriterAppender extends AppenderSkeleton {
   }
 
   /**
-     Close the underlying {@link java.io.Writer}.
-  */
-  protected 
-  void closeWriter() {
+   * Close the underlying {@link java.io.Writer}.
+   * */
+  protected void closeWriter() {
     if(qw != null) {
       try {
 	qw.close();
       } catch(IOException e) {
-	LogLog.error("Could not close " + qw, e); // do need to invoke an error handler
-	                                          // at  this late stage
+	// There is do need to invoke an error handler at this late
+	// stage.
+	LogLog.error("Could not close " + qw, e);
       }
     }
   }
 
-
   /**
-     Retuns the option names for this component.
+     Returns an OutputStreamWriter when passed an OutputStream.  The
+     encoding used will depend on the value of the
+     <code>encoding</code> property.  If the encoding value is
+     specified incorrectly the writer will be opened using the default
+     system encoding (an error message will be printed to the loglog.  */
+  protected
+  OutputStreamWriter createWriter(OutputStream os) {
+    OutputStreamWriter retval = null;
 
-     @deprecated We now use JavaBeans introspection to configure
-     components. Options strings are no longer needed.
-  */
-  public
-  String[] getOptionStrings() {
-    return OptionConverter.concatanateArrays(super.getOptionStrings(),
-           new String[] {IMMEDIATE_FLUSH_OPTION});
+    String enc = getEncoding();
+    if(enc != null) {
+      try {
+	retval = new OutputStreamWriter(os, enc);
+      } catch(IOException e) {
+	LogLog.warn("Error initializing output writer.");
+	LogLog.warn("Unsupported encoding?");
+      }
+    }
+    if(retval == null) {
+      retval = new OutputStreamWriter(os);
+    }
+    return retval;
   }
 
-  
+  public String getEncoding() {
+    return encoding;
+  }
+
+  public void setEncoding(String value) {
+    encoding = value;
+  }
+
+
+
+
   /**
-     Set the {@link ErrorHandler} for this FileAppender and also the
+     Set the {@link ErrorHandler} for this WriterAppender and also the
      underlying {@link QuietWriter} if any. */
-  public
-  synchronized 
-  void setErrorHandler(ErrorHandler eh) {
+  public synchronized void setErrorHandler(ErrorHandler eh) {
     if(eh == null) {
       LogLog.warn("You have tried to set a null error-handler.");
     } else {
@@ -244,38 +257,22 @@ public class WriterAppender extends AppenderSkeleton {
       if(this.qw != null) {
 	this.qw.setErrorHandler(eh);
       }
-    }    
+    }
   }
 
-  /**
-     @deprecated Use the setter method for the option directly instead
-     of the generic <code>setOption</code> method. 
-   */
- public
-  void setOption(String key, String value) {
-    if(value == null) return;
-    super.setOption(key, value);
-    
-    if (key.equalsIgnoreCase(IMMEDIATE_FLUSH_OPTION)) {
-      immediateFlush = OptionConverter.toBoolean(value, immediateFlush);
-    }
-  }  
-  
   /**
     <p>Sets the Writer where the log output will go. The
     specified Writer must be opened by the user and be
     writable.
-    
+
     <p>The <code>java.io.Writer</code> will be closed when the
     appender instance is closed.
 
-    
+
     <p><b>WARNING:</b> Logging to an unopened Writer will fail.
-    <p>  
-    @param Writer An already opened Writer.  */
-  public
-  synchronized
-  void setWriter(Writer writer) {
+    <p>
+    @param writer An already opened Writer.  */
+  public synchronized void setWriter(Writer writer) {
     reset();
     this.qw = new QuietWriter(writer, errorHandler);
     //this.tp = new TracerPrintWriter(qw);
@@ -304,10 +301,10 @@ public class WriterAppender extends AppenderSkeleton {
 	}
       }
     }
- 
+
     if(this.immediateFlush) {
       this.qw.flush();
-    } 
+    }
   }
 
 
@@ -330,7 +327,7 @@ public class WriterAppender extends AppenderSkeleton {
   void reset() {
     closeWriter();
     this.qw = null;
-    //this.tp = null;    
+    //this.tp = null;
   }
 
 
@@ -341,15 +338,17 @@ public class WriterAppender extends AppenderSkeleton {
   void writeFooter() {
     if(layout != null) {
       String f = layout.getFooter();
-      if(f != null && this.qw != null)
+      if(f != null && this.qw != null) {
 	this.qw.write(f);
+	this.qw.flush();
+      }
     }
   }
 
-/**
+  /**
      Write a header as produced by the embedded layout's {@link
      Layout#getHeader} method.  */
-  protected 
+  protected
   void writeHeader() {
     if(layout != null) {
       String h = layout.getHeader();
