@@ -1,10 +1,11 @@
 /*
  * Copyright (C) The Apache Software Foundation. All rights reserved.
  *
- * This software is published under the terms of the Apache Software License
- * version 1.1, a copy of which has been included  with this distribution in
- * the LICENSE.APL file.
- */
+ * This software is published under the terms of the Apache Software
+ * License version 1.1, a copy of which has been included with this
+ * distribution in the LICENSE.txt file.  */
+
+// Contributors:  Georg Lundesgaard
 
 package org.apache.log4j.config;
 
@@ -12,7 +13,6 @@ import java.beans.Introspector;
 import java.beans.PropertyDescriptor;
 import java.beans.BeanInfo;
 import java.beans.IntrospectionException;
-import java.io.*;
 import java.lang.reflect.*;
 import java.util.*;
 import org.apache.log4j.*;
@@ -98,7 +98,7 @@ public class PropertySetter {
   void setProperties(Properties properties, String prefix) {
     int len = prefix.length();
     
-    for (Enumeration e = properties.keys(); e.hasMoreElements(); ) {
+    for (Enumeration e = properties.propertyNames(); e.hasMoreElements(); ) {
       String key = (String) e.nextElement();
       
       // handle only properties that start with the desired frefix.
@@ -154,8 +154,8 @@ public class PropertySetter {
       try {
         setProperty(prop, name, value);
       } catch (PropertySetterException ex) {
-        LogLog.warn("Failed to set property " + name +
-                    " to value \"" + value + "\". " + ex.getMessage());
+        LogLog.warn("Failed to set property [" + name +
+                    "] to value \"" + value + "\". ", ex.rootCause);
       }
     }
   }
@@ -173,7 +173,7 @@ public class PropertySetter {
     throws PropertySetterException {
     Method setter = prop.getWriteMethod();
     if (setter == null) {
-      throw new PropertySetterException("No setter for property");
+      throw new PropertySetterException("No setter for property ["+name+"].");
     }
     Class[] paramTypes = setter.getParameterTypes();
     if (paramTypes.length != 1) {
@@ -223,7 +223,7 @@ public class PropertySetter {
         return Boolean.FALSE;
       }
     } else if (Priority.class.isAssignableFrom(type)) {
-      return OptionConverter.toPriority(v, Priority.DEBUG);
+      return OptionConverter.toLevel(v, (Level) Level.DEBUG);
     }
     return null;
   }
