@@ -3,27 +3,24 @@
  *
  * This software is published under the terms of the Apache Software
  * License version 1.1, a copy of which has been included with this
- * distribution in the LICENSE.APL file.  */
+ * distribution in the LICENSE.txt file.  */
 
 package org.apache.log4j.nt.test;
 
 
-import org.apache.log4j.Category;
-import org.apache.log4j.Layout;
+import org.apache.log4j.Logger;
 import org.apache.log4j.BasicConfigurator;
-import org.apache.log4j.Appender;
 import org.apache.log4j.nt.NTEventLogAppender;
 import org.apache.log4j.Priority;
 import org.apache.log4j.NDC;
-import java.io.IOException;
 
 
 public class NTMin {
-  
-  static Category cat = Category.getInstance(NTMin.class.getName());
-				     
-  public 
-  static 
+
+  static Logger cat = Logger.getLogger(NTMin.class.getName());
+
+  public
+  static
   void main(String argv[]) {
 
     //if(argv.length == 1) {
@@ -35,7 +32,7 @@ public class NTMin {
       test("someHost");
   }
 
-  
+
   static
   void Usage(String msg) {
     System.err.println(msg);
@@ -43,7 +40,7 @@ public class NTMin {
     System.exit(1);
   }
 
-  
+
   static
   void init() {
 
