@@ -3,11 +3,11 @@
  *
  * This software is published under the terms of the Apache Software
  * License version 1.1, a copy of which has been included with this
- * distribution in the LICENSE.APL file.  */
+ * distribution in the LICENSE.txt file.  */
 
 package org.apache.log4j.spi;
 
-import org.apache.log4j.Hierarchy;
+import org.apache.log4j.spi.LoggerRepository;
 import java.net.URL;
 
 /**
@@ -17,6 +17,21 @@ import java.net.URL;
    @author Anders Kristensen
  */
 public interface Configurator {
+
+  /**
+     Special level value signifying inherited behaviour. The current
+     value of this string constant is <b>inherited</b>. {@link #NULL}
+     is a synonym.  */
+  public static final String INHERITED = "inherited";
+
+  /**
+     Special level signifying inherited behaviour, same as {@link
+     #INHERITED}. The current value of this string constant is
+     <b>null</b>. */
+  public static final String NULL = "null";
+
+
+
   /**
      Interpret a resource pointed by a URL and set up log4j accordingly.
 
@@ -24,7 +39,7 @@ public interface Configurator {
      parameter.
 
      @param url The URL to parse
-     @param hierarchy The hierarchy to operation upon.
+     @param repository The hierarchy to operation upon.
    */
-  void doConfigure(URL url, Hierarchy hierarchy);
+  void doConfigure(URL url, LoggerRepository repository);
 }
