@@ -3,7 +3,7 @@
  *
  * This software is published under the terms of the Apache Software
  * License version 1.1, a copy of which has been included with this
- * distribution in the LICENSE.APL file.  */
+ * distribution in the LICENSE.txt file.  */
 
 package org.apache.log4j.xml;
 
@@ -13,30 +13,24 @@ import java.net.URL;
 
 import org.w3c.dom.*;
 import java.lang.reflect.Method;
-import org.apache.log4j.Category;
-import org.apache.log4j.spi.OptionHandler;
-import org.apache.log4j.spi.ErrorHandler;
-import org.apache.log4j.spi.AppenderAttachable;
-import org.apache.log4j.spi.Configurator;
-import org.apache.log4j.Appender;
-import org.apache.log4j.Layout;
-import org.apache.log4j.Priority;
-import org.apache.log4j.Hierarchy;
-import org.apache.log4j.BasicConfigurator;
-import org.apache.log4j.helpers.LogLog;
-import org.apache.log4j.spi.Filter;
+import org.apache.log4j.*;
+import org.apache.log4j.spi.*;
+import org.apache.log4j.or.RendererMap;
+import org.apache.log4j.helpers.*;
 import org.apache.log4j.config.PropertySetter;
-import org.apache.log4j.helpers.OptionConverter;
-import org.apache.log4j.helpers.FileWatchdog;
+
 import org.xml.sax.InputSource;
 import java.io.FileInputStream;
 import java.io.InputStream;
+import java.io.Reader;
 import java.io.IOException;
 import java.net.URL;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.FactoryConfigurationError;
 
+// Contributors:   Mark Womack
+//                 Arun Katkere 
 
 /**
    Use this class to initialize the log4j environment using a DOM tree.
@@ -50,9 +44,9 @@ import javax.xml.parsers.FactoryConfigurationError;
    line. Alternatively, set the <code>debug</code> attribute in the
    <code>log4j:configuration</code> element. As in
 <pre>
-   <log4j:configuration debug=true" xmlns:log4j="http://jakarta.apache.org/log4j/">
+   &lt;log4j:configuration <b>debug="true"</b> xmlns:log4j="http://jakarta.apache.org/log4j/">
    ...
-   </log4j:configuration>
+   &lt;/log4j:configuration>
 </pre>
 
    <p>There are sample XML files included in the package.
@@ -62,7 +56,7 @@ import javax.xml.parsers.FactoryConfigurationError;
    @author Anders Kristensen
 
    @since 0.8.3 */
-public class DOMConfigurator extends BasicConfigurator implements Configurator {
+public class DOMConfigurator implements Configurator {
 
   static final String CONFIGURATION_TAG = "log4j:configuration";
   static final String OLD_CONFIGURATION_TAG = "configuration";
@@ -72,17 +66,21 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   static final String PARAM_TAG    	= "param";
   static final String LAYOUT_TAG	= "layout";
   static final String CATEGORY		= "category";
+  static final String LOGGER		= "logger";
+  static final String LOGGER_REF	= "logger-ref";
+  static final String CATEGORY_FACTORY_TAG  = "categoryFactory";
   static final String NAME_ATTR		= "name";
   static final String CLASS_ATTR        = "class";
   static final String VALUE_ATTR	= "value";
   static final String ROOT_TAG		= "root";
-  static final String PRIORITY_TAG	= "priority";
+  static final String ROOT_REF		= "root-ref";
+  static final String LEVEL_TAG	        = "level";
+  static final String PRIORITY_TAG      = "priority";
   static final String FILTER_TAG	= "filter";
   static final String ERROR_HANDLER_TAG	= "errorHandler";
   static final String REF_ATTR		= "ref";
   static final String ADDITIVITY_ATTR    = "additivity";  
-  static final String DISABLE_OVERRIDE_ATTR = "disableOverride";
-  static final String DISABLE_ATTR       = "disable";
+  static final String THRESHOLD_ATTR       = "threshold";
   static final String CONFIG_DEBUG_ATTR  = "configDebug";
   static final String INTERNAL_DEBUG_ATTR  = "debug";
   static final String RENDERING_CLASS_ATTR = "renderingClass";
@@ -98,6 +96,7 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   Hashtable appenderBag;
 
   Properties props;
+  LoggerRepository repository;
 
   /**
      No argument constructor.
@@ -108,25 +107,22 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   }
 
   /**
-     Used internally to parse appenders by IDREF.
-   */
+     Used internally to parse appenders by IDREF name.
+  */
   protected
-  Appender findAppenderByReference(Element appenderRef) {    
-    String appenderName = subst(appenderRef.getAttribute(REF_ATTR));
-
+  Appender findAppenderByName(Document doc, String appenderName)  {      
     Appender appender = (Appender) appenderBag.get(appenderName);
+
     if(appender != null) {
       return appender;
     } else {
-      Document doc = appenderRef.getOwnerDocument();
-
       // Doesn't work on DOM Level 1 :
       // Element element = doc.getElementById(appenderName);
                         
       // Endre's hack:
       Element element = null;
       NodeList list = doc.getElementsByTagName("appender");
-      for (int t=0; t<list.getLength(); t++) {
+      for (int t=0; t < list.getLength(); t++) {
 	Node node = list.item(t);
 	NamedNodeMap map= node.getAttributes();
 	Node attrNode = map.getNamedItem("name");
@@ -147,6 +143,15 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
       }
     } 
   }
+  /**
+     Used internally to parse appenders by IDREF element.
+   */
+  protected
+  Appender findAppenderByReference(Element appenderRef) {    
+    String appenderName = subst(appenderRef.getAttribute(REF_ATTR));    
+    Document doc = appenderRef.getOwnerDocument();
+    return findAppenderByName(doc, appenderName);
+  }
 
   /**
      Used internally to parse an appender element.
@@ -156,7 +161,7 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
     String className = subst(appenderElement.getAttribute(CLASS_ATTR));
     LogLog.debug("Class name: [" + className+']');    
     try {
-      Object instance 	= Class.forName(className).newInstance();
+      Object instance 	= Loader.loadClass(className).newInstance();
       Appender appender	= (Appender)instance;
       PropertySetter propSetter = new PropertySetter(appender);
 
@@ -225,6 +230,8 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
  				       null);
     
     if(eh != null) {
+      eh.setAppender(appender);
+
       PropertySetter propSetter = new PropertySetter(eh);
       NodeList children = element.getChildNodes();
       final int length 	= children.getLength();
@@ -236,6 +243,15 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
 	  String tagName = currentElement.getTagName();
 	  if(tagName.equals(PARAM_TAG)) {
             setParameter(currentElement, propSetter);
+	  } else if(tagName.equals(APPENDER_REF_TAG)) {
+	    eh.setBackupAppender(findAppenderByReference(currentElement));
+	  } else if(tagName.equals(LOGGER_REF)) {
+	    String loggerName = currentElement.getAttribute(REF_ATTR);	    
+	    Logger logger = repository.getLogger(loggerName);
+	    eh.setLogger(logger);
+	  } else if(tagName.equals(ROOT_REF)) {
+	    Logger root = repository.getRootLogger();
+	    eh.setLogger(root);
 	  }
 	}
       }
@@ -265,10 +281,12 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
 	  String tagName = currentElement.getTagName();
 	  if(tagName.equals(PARAM_TAG)) {
             setParameter(currentElement, propSetter);
-	  }
+	  } 
 	}
       }
       propSetter.activate();
+      LogLog.debug("Adding filter of type ["+filter.getClass()
+		   +"] to appender named ["+appender.getName()+"].");
       appender.addFilter(filter);
     }    
   }
@@ -277,26 +295,26 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
      Used internally to parse an category element.
   */
   protected
-  void parseCategory (Element categoryElement, Hierarchy hierarchy) {
+  void parseCategory (Element loggerElement) {
     // Create a new org.apache.log4j.Category object from the <category> element.
-    String catName = subst(categoryElement.getAttribute(NAME_ATTR));
+    String catName = subst(loggerElement.getAttribute(NAME_ATTR));
 
-    Category cat;    
+    Logger cat;    
 
-    String className = subst(categoryElement.getAttribute(CLASS_ATTR));
+    String className = subst(loggerElement.getAttribute(CLASS_ATTR));
 
 
     if(EMPTY_STR.equals(className)) {
-      LogLog.debug("Retreiving an instance of org.apache.log4j.Category.");
-      cat = hierarchy.getInstance(catName);
+      LogLog.debug("Retreiving an instance of org.apache.log4j.Logger.");
+      cat = repository.getLogger(catName);
     }
     else {
-      LogLog.debug("Desired category sub-class: ["+className+']');
+      LogLog.debug("Desired logger sub-class: ["+className+']');
        try {	 
-	 Class clazz = Class.forName(className);
-	 Method getInstanceMethod = clazz.getMethod("getInstance", 
+	 Class clazz = Loader.loadClass(className);
+	 Method getInstanceMethod = clazz.getMethod("getLogger", 
 						    ONE_STRING_PARAM);
-	 cat = (Category) getInstanceMethod.invoke(null, new Object[] {catName});
+	 cat = (Logger) getInstanceMethod.invoke(null, new Object[] {catName});
        } catch (Exception oops) {
 	 LogLog.error("Could not retrieve category ["+catName+
 		      "]. Reported error follows.", oops);
@@ -309,12 +327,48 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
     // configuration is in progress.
     synchronized(cat) {
       boolean additivity = OptionConverter.toBoolean(
-                           subst(categoryElement.getAttribute(ADDITIVITY_ATTR)),
+                           subst(loggerElement.getAttribute(ADDITIVITY_ATTR)),
 			   true);
     
       LogLog.debug("Setting ["+cat.getName()+"] additivity to ["+additivity+"].");
       cat.setAdditivity(additivity);
-      parseChildrenOfCategoryElement(categoryElement, cat, false);
+      parseChildrenOfLoggerElement(loggerElement, cat, false);
+    }
+  }
+
+
+  /**
+     Used internally to parse the category factory element.
+  */
+  protected
+  void parseCategoryFactory(Element factoryElement) {
+    String className = subst(factoryElement.getAttribute(CLASS_ATTR));
+
+    if(EMPTY_STR.equals(className)) {
+      LogLog.error("Category Factory tag " + CLASS_ATTR + " attribute not found.");
+      LogLog.debug("No Category Factory configured.");
+    }
+    else {
+      LogLog.debug("Desired category factory: ["+className+']');
+      Object catFactory = OptionConverter.instantiateByClassName(className, 
+                                                                 LoggerFactory.class, 
+                                                                 null);
+      PropertySetter propSetter = new PropertySetter(catFactory);
+
+      Element  currentElement = null;
+      Node     currentNode    = null;
+      NodeList children       = factoryElement.getChildNodes();
+      final int length        = children.getLength();
+
+      for (int loop=0; loop < length; loop++) {
+        currentNode = children.item(loop);
+	if (currentNode.getNodeType() == Node.ELEMENT_NODE) {
+	  currentElement = (Element)currentNode;
+	  if (currentElement.getTagName().equals(PARAM_TAG)) {
+	    setParameter(currentElement, propSetter);
+	  }
+	}
+      }
     }
   }
 
@@ -323,11 +377,11 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
      Used internally to parse the roor category element.
   */
   protected
-  void parseRoot (Element rootElement, Hierarchy hierarchy) {
-    Category root = hierarchy.getRoot();
+  void parseRoot (Element rootElement) {
+    Logger root = repository.getRootLogger();
     // category configuration needs to be atomic
     synchronized(root) {    
-      parseChildrenOfCategoryElement(rootElement, root, true);
+      parseChildrenOfLoggerElement(rootElement, root, true);
     }
   }
 
@@ -336,8 +390,8 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
      Used internally to parse the children of a category element.
   */
   protected
-  void parseChildrenOfCategoryElement(Element catElement,
-				      Category cat, boolean isRoot) {
+  void parseChildrenOfLoggerElement(Element catElement,
+				      Logger cat, boolean isRoot) {
     
     PropertySetter propSetter = new PropertySetter(cat);
     
@@ -368,8 +422,10 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
 	    
 	  cat.addAppender(appender);
 	  
+	} else if(tagName.equals(LEVEL_TAG)) {
+	  parseLevel(currentElement, cat, isRoot);	
 	} else if(tagName.equals(PRIORITY_TAG)) {
-	  parsePriority(currentElement, cat, isRoot);	
+	  parseLevel(currentElement, cat, isRoot);
 	} else if(tagName.equals(PARAM_TAG)) {
           setParameter(currentElement, propSetter);
 	}
@@ -386,7 +442,7 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
     String className = subst(layout_element.getAttribute(CLASS_ATTR));
     LogLog.debug("Parsing layout of class: \""+className+"\"");		 
     try {
-      Object instance 	= Class.forName(className).newInstance();
+      Object instance 	= Loader.loadClass(className).newInstance();
       Layout layout   	= (Layout)instance;
       PropertySetter propSetter = new PropertySetter(layout);
       
@@ -415,54 +471,55 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   }
 
   protected 
-  void parseRenderer(Element element, Hierarchy hierarchy) {
+  void parseRenderer(Element element) {
     String renderingClass = subst(element.getAttribute(RENDERING_CLASS_ATTR));
     String renderedClass = subst(element.getAttribute(RENDERED_CLASS_ATTR));
-    addRenderer(hierarchy, renderedClass, renderingClass);
+    if(repository instanceof RendererSupport) {
+      RendererMap.addRenderer((RendererSupport) repository, renderedClass, 
+			      renderingClass);
+    }
   }
 
   /**
-     Used internally to parse a priority  element.
+     Used internally to parse a level  element.
   */
   protected
-  void parsePriority(Element element, Category cat, boolean isRoot) {
-    String catName = cat.getName();
+  void parseLevel(Element element, Logger logger, boolean isRoot) {
+    String catName = logger.getName();
     if(isRoot) {
       catName = "root";
     }
 
     String priStr = subst(element.getAttribute(VALUE_ATTR));
-    LogLog.debug("Priority value for "+catName+" is  ["+priStr+"].");
+    LogLog.debug("Level value for "+catName+" is  ["+priStr+"].");
     
-    if(BasicConfigurator.INHERITED.equals(priStr)) {
+    if(INHERITED.equalsIgnoreCase(priStr) || NULL.equalsIgnoreCase(priStr)) {
       if(isRoot) {
-	LogLog.error("Root priority cannot be inherited. Ignoring directive.");
+	LogLog.error("Root level cannot be inherited. Ignoring directive.");
       } else {
-	cat.setPriority(null);
+	logger.setLevel(null);
       }
-    }
-    else {
+    } else {
       String className = subst(element.getAttribute(CLASS_ATTR));      
-      if(EMPTY_STR.equals(className)) {      
-	
-	cat.setPriority(Priority.toPriority(priStr));
+      if(EMPTY_STR.equals(className)) {	
+	logger.setLevel(OptionConverter.toLevel(priStr, Level.DEBUG));
       } else {
-	LogLog.debug("Desired Priority sub-class: ["+className+']');
+	LogLog.debug("Desired Level sub-class: ["+className+']');
 	try {	 
-	  Class clazz = Class.forName(className);
-	  Method toPriorityMethod = clazz.getMethod("toPriority", 
+	  Class clazz = Loader.loadClass(className);
+	  Method toLevelMethod = clazz.getMethod("toLevel", 
 						    ONE_STRING_PARAM);
-	  Priority pri = (Priority) toPriorityMethod.invoke(null, 
+	  Level pri = (Level) toLevelMethod.invoke(null, 
 						    new Object[] {priStr});
-	  cat.setPriority(pri);
+	  logger.setLevel(pri);
 	} catch (Exception oops) {
-	  LogLog.error("Could not create priority ["+priStr+
+	  LogLog.error("Could not create level ["+priStr+
 		       "]. Reported error follows.", oops);
 	  return;
 	}
       }
     }
-    LogLog.debug(catName + " priority set to " + cat.getPriority());    
+    LogLog.debug(catName + " level set to " + logger.getLevel());    
   }
 
   protected
@@ -483,7 +540,7 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   public
   void configure (Element element) {
     DOMConfigurator configurator = new DOMConfigurator();
-    configurator.parse(element, Category.getDefaultHierarchy());
+    configurator.doConfigure(element,  LogManager.getLoggerRepository());
   }
 
  /**
@@ -520,11 +577,11 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   }
 
   public
-  void doConfigure(String filename, Hierarchy hierarchy) {
+  void doConfigure(String filename, LoggerRepository repository) {
     FileInputStream fis = null;
     try {
       fis = new FileInputStream(filename);
-      doConfigure(fis, hierarchy);
+      doConfigure(fis, repository);
     } catch(IOException e) {
       LogLog.error("Could not open ["+filename+"].", e);
     } finally {
@@ -540,9 +597,9 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
   
 
   public
-  void doConfigure(URL url, Hierarchy hierarchy) {
+  void doConfigure(URL url, LoggerRepository repository) {
     try {
-      doConfigure(url.openStream(), hierarchy);
+      doConfigure(url.openStream(), repository);
     } catch(IOException e) {
       LogLog.error("Could not open ["+url+"].", e);
     }
@@ -554,12 +611,36 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
 
   */
   public
-  void doConfigure(InputStream inputStream, Hierarchy hierarchy) 
+  void doConfigure(InputStream inputStream, LoggerRepository repository) 
+                                          throws FactoryConfigurationError {
+    doConfigure(new InputSource(inputStream), repository);
+  }
+
+  /**
+     Configure log4j by reading in a log4j.dtd compliant XML
+     configuration file.
+
+  */
+  public
+  void doConfigure(Reader reader, LoggerRepository repository) 
+                                          throws FactoryConfigurationError {
+    doConfigure(new InputSource(reader), repository);
+  }
+
+  /**
+     Configure log4j by reading in a log4j.dtd compliant XML
+     configuration file.
+
+  */
+  protected
+  void doConfigure(InputSource inputSource, LoggerRepository repository) 
                                           throws FactoryConfigurationError {
     DocumentBuilderFactory dbf = null;
+    this.repository = repository;
     try { 
       LogLog.debug("System property is :"+
-  	                        OptionConverter.getSystemProperty(dbfKey, null)); 
+  	                        OptionConverter.getSystemProperty(dbfKey, 
+								  null)); 
       dbf = DocumentBuilderFactory.newInstance();
       LogLog.debug("Standard DocumentBuilderFactory search succeded.");
       LogLog.debug("DocumentBuilderFactory is: "+dbf.getClass().getName());
@@ -570,49 +651,50 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
     }
       
     try {
-      // This makes ID/IDREF attributes to have a meaning. Don't ask
-      // me why.
       dbf.setValidating(true);
-      //dbf.setNamespaceAware(true);
 
       DocumentBuilder docBuilder = dbf.newDocumentBuilder();
-      //docBuilder.setErrorHandler(new ReportParserError());
 
-      InputSource inputSource = new InputSource(inputStream);
-      Class clazz = this.getClass();
-      URL dtdURL = clazz.getResource("/org/apache/log4j/xml/log4j.dtd");
-      if(dtdURL == null) {
-	LogLog.error("Could not find [log4j.dtd]. Used ["+clazz.getClassLoader()+
-		     "] class loader in the search.");
-      }
-      else {
-	LogLog.debug("URL to log4j.dtd is [" + dtdURL.toString()+"].");
-	inputSource.setSystemId(dtdURL.toString());
-      }
-      Document doc = docBuilder.parse(inputSource);
-      parse(doc.getDocumentElement(), hierarchy);
+      docBuilder.setErrorHandler(new SAXErrorHandler());      
+      docBuilder.setEntityResolver(new Log4jEntityResolver());        
+      // we change the system ID to a valid URI so that Crimson won't
+      // complain. Indeed, "log4j.dtd" alone is not a valid URI which
+      // causes Crimson to barf. The Log4jEntityResolver only cares
+      // about the "log4j.dtd" ending.
+      inputSource.setSystemId("dummy://log4j.dtd");
+      Document doc = docBuilder.parse(inputSource); 
+      parse(doc.getDocumentElement());
     } catch (Exception e) {
       // I know this is miserable...
-      LogLog.error("Could not parse input stream ["+inputStream+"].", e);
+      LogLog.error("Could not parse input source ["+inputSource+"].", e);
     }
   }
 
+  /**
+     Configure by taking in an DOM element. 
+  */
+  public void doConfigure(Element element, LoggerRepository repository) {
+    this.repository = repository;
+    parse(element);
+  }
+
   
   /**
-     A static version of {@link #doConfigure(String, Hierarchy)}.  */
+     A static version of {@link #doConfigure(String, LoggerRepository)}.  */
   static
   public
   void configure(String filename) throws FactoryConfigurationError {
-    new DOMConfigurator().doConfigure(filename, Category.getDefaultHierarchy());
+    new DOMConfigurator().doConfigure(filename, 
+				      LogManager.getLoggerRepository());
   }
 
   /**
-     A static version of {@link #doConfigure(URL, Hierarchy)}.
+     A static version of {@link #doConfigure(URL, LoggerRepository)}.
    */
   static
   public
   void configure(URL url) throws FactoryConfigurationError {
-    new DOMConfigurator().doConfigure(url, Category.getDefaultHierarchy());
+    new DOMConfigurator().doConfigure(url, LogManager.getLoggerRepository());
   }
 
   /**
@@ -622,7 +704,7 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
      
   */
   protected
-  void parse(Element element, Hierarchy hierarchy) {
+  void parse(Element element) {
 
     String rootElementName = element.getTagName();
 
@@ -644,9 +726,9 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
     // "debug" attribute is returned as the empty string.
     if(!debugAttrib.equals("") && !debugAttrib.equals("null")) {      
       LogLog.setInternalDebugging(OptionConverter.toBoolean(debugAttrib, true));
-    }
-    else 
+    } else {
       LogLog.debug("Ignoring " + INTERNAL_DEBUG_ATTR + " attribute.");
+    }
 
 
     String confDebug = subst(element.getAttribute(CONFIG_DEBUG_ATTR));
@@ -656,43 +738,51 @@ public class DOMConfigurator extends BasicConfigurator implements Configurator {
       LogLog.setInternalDebugging(OptionConverter.toBoolean(confDebug, true));
     }
 
-    String override = subst(element.getAttribute(DISABLE_OVERRIDE_ATTR));
-    LogLog.debug("Disable override=\"" + override +"\".");
-    // if the log4j.dtd is not specified in the XML file, then the
-    // DISABLE_OVERRIDE attribute is returned as the empty string when
-    // it is not specified in the XML file.
-    if(!override.equals("") && !override.equals("null")) {
-      hierarchy.overrideAsNeeded(override);
+    String thresholdStr = subst(element.getAttribute(THRESHOLD_ATTR));
+    LogLog.debug("Threshold =\"" + thresholdStr +"\".");
+    if(!"".equals(thresholdStr) && !"null".equals(thresholdStr)) {
+      repository.setThreshold(thresholdStr);
     }
 
-    String disableStr = subst(element.getAttribute(DISABLE_ATTR));
-    LogLog.debug("Disable =\"" + disableStr +"\".");
-    if(!"".equals(disableStr) && !"null".equals(disableStr)) {
-      hierarchy.disable(disableStr);
-    }
-    
-
-
-
     //Hashtable appenderBag = new Hashtable(11);
 
     /* Building Appender objects, placing them in a local namespace
        for future reference */
+
+    // First configure each category factory under the root element.
+    // Category factories need to be configured before any of
+    // categories they support.
+    //
+    String   tagName = null;
+    Element  currentElement = null;
+    Node     currentNode = null;
     NodeList children = element.getChildNodes();
     final int length = children.getLength();
+
+    for (int loop = 0; loop < length; loop++) {
+      currentNode = children.item(loop);
+      if (currentNode.getNodeType() == Node.ELEMENT_NODE) {
+	currentElement = (Element) currentNode;
+	tagName = currentElement.getTagName();
+
+	if (tagName.equals(CATEGORY_FACTORY_TAG)) {
+	  parseCategoryFactory(currentElement);
+	}
+      }
+    }
     
     for (int loop = 0; loop < length; loop++) {
-      Node currentNode = children.item(loop);
+      currentNode = children.item(loop);
       if (currentNode.getNodeType() == Node.ELEMENT_NODE) {
-	Element currentElement = (Element) currentNode;
-	String tagName = currentElement.getTagName();
+	currentElement = (Element) currentNode;
+	tagName = currentElement.getTagName();
 
-	if (tagName.equals(CATEGORY)) {
-	  parseCategory(currentElement, hierarchy);
+	if (tagName.equals(CATEGORY) || tagName.equals(LOGGER)) {
+	  parseCategory(currentElement);
 	} else if (tagName.equals(ROOT_TAG)) {
-	  parseRoot(currentElement, hierarchy);
+	  parseRoot(currentElement);
 	} else if(tagName.equals(RENDERER_TAG)) {
-	  parseRenderer(currentElement, hierarchy);
+	  parseRenderer(currentElement);
 	}
       }
     }
@@ -722,6 +812,7 @@ class XMLWatchdog extends FileWatchdog {
      <code>filename</code> to reconfigure log4j. */
   public
   void doOnChange() {
-    new DOMConfigurator().doConfigure(filename, Category.getDefaultHierarchy());
+    new DOMConfigurator().doConfigure(filename, 
+				      LogManager.getLoggerRepository());
   }
 }
