@@ -2,8 +2,7 @@
 package org.apache.log4j.xml.examples;
 
 import org.apache.log4j.xml.DOMConfigurator;
-import org.apache.log4j.Category;
-import org.apache.log4j.Priority;
+import org.apache.log4j.Logger;
 import java.net.*;
 
 /**
@@ -14,24 +13,24 @@ import java.net.*;
    <p>Sample XML files <a href="doc-files/sample1.xml">sample1.xml</a>
    and <a href="doc-files/sample2.xml">sample2.xml</a> are provided.
 
-   
+
    <p>Note that the log4j.dtd is not in the local directory.
    It is found by the class loader.
-   
+
    @author Ceki G&uuml;lc&uuml;
 
 */
 public class XMLSample {
-  
-  static Category cat = Category.getInstance(XMLSample.class.getName());
 
-  public 
-  static 
+  static Logger cat = Logger.getLogger(XMLSample.class);
+
+  public
+  static
   void main(String argv[]) {
 
-    if(argv.length == 1) 
+    if(argv.length == 1)
       init(argv[0]);
-    else 
+    else
       Usage("Wrong number of arguments.");
     sample();
   }
@@ -43,7 +42,7 @@ public class XMLSample {
 			"configFile");
     System.exit(1);
   }
-  
+
   static
   void init(String configFile) {
     DOMConfigurator.configure(configFile);
@@ -52,10 +51,10 @@ public class XMLSample {
   static
   void sample() {
     int i = -1;
-    Category root = Category.getRoot();    
+    Logger root = Logger.getRootLogger();
     cat.debug("Message " + ++i);
     cat.warn ("Message " + ++i);
-    cat.error("Message " + ++i);        
+    cat.error("Message " + ++i);
     Exception e = new Exception("Just testing");
     cat.debug("Message " + ++i, e);
   }
