@@ -8,8 +8,9 @@
 package org.apache.log4j.xml.test;
 
 import org.apache.log4j.xml.DOMConfigurator;
-import org.apache.log4j.Category;
-import org.apache.log4j.Priority;
+import org.apache.log4j.Logger;
+import org.apache.log4j.LogManager;
+import org.apache.log4j.Level;
 //import org.apache.log4j.xml.examples.ReportParserError;
 //import org.apache.xerces.parsers.DOMParser;
 //import java.io.FileInputStream;
@@ -19,7 +20,7 @@ import org.apache.log4j.Priority;
    @author Ceki G&uuml;lc&uuml;
 */
 public class DOMTest {
-  static Category cat = Category.getInstance(DOMTest.class.getName());
+  static Logger cat = Logger.getLogger(DOMTest.class.getName());
 
 
   public 
@@ -50,7 +51,7 @@ public class DOMTest {
   static
   void test() {
     int i = -1;
-    Category root = Category.getRoot();
+    Logger root = Logger.getRootLogger();
     
     cat.debug("Message " + ++i);
     root.debug("Message " + i);        
@@ -64,8 +65,8 @@ public class DOMTest {
     cat.error("Message " + ++i);
     root.error("Message " + i);
     
-    cat.log(Priority.FATAL, "Message " + ++i);
-    root.log(Priority.FATAL, "Message " + i);    
+    cat.log(Level.FATAL, "Message " + ++i);
+    root.log(Level.FATAL, "Message " + i);    
     
     Exception e = new Exception("Just testing");
     cat.debug("Message " + ++i, e);
@@ -74,6 +75,6 @@ public class DOMTest {
     cat.error("Message " + ++i, e);
     root.error("Message " + i, e);    
 
-    Category.shutdown();
+    LogManager.shutdown();
   }
 }
